package co.hyperverge.hyperkyc_flutter;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;

import androidx.annotation.NonNull;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import co.hyperverge.hyperkyc.HyperKyc;
import co.hyperverge.hyperkyc.data.models.HyperKycConfig;
import co.hyperverge.hyperkyc.data.models.HyperKycData;
import co.hyperverge.hyperkyc.data.models.HyperKycResult;
import co.hyperverge.hyperkyc.data.network.BaseResponse;
import co.hyperverge.hyperkyc.data.network.DocCaptureApiDetail;
import co.hyperverge.hyperkyc.data.network.FaceCaptureApiDetail;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;

/**
 * HyperkycFlutterPlugin
 */
public class HyperkycFlutterPlugin implements FlutterPlugin, ActivityAware, PluginRegistry.ActivityResultListener {
    private static final String TAG = HyperkycFlutterPlugin.class.getCanonicalName();
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private MethodChannel hyperKycChannel;
    protected Context context;
    protected Activity activity;
    private Result pendingResult;
    private final FLHyperKyc flHyperKyc = new FLHyperKyc();
    private final HyperKyc.Contract contract = new HyperKyc.Contract();
    private HyperKycConfig hyperKycConfig;

    private final MethodCallHandler hyperKycMethodCallHandler = (@NonNull MethodCall call, @NonNull Result result) -> {
        pendingResult = result;
        if (call.method.equalsIgnoreCase(FLHyperKycConstants.LAUNCH)) {
            HashMap<String, Object> hyperKycConfigMap = call.argument(FLHyperKycConstants.HYPERKYCCONFIG);
            assert hyperKycConfigMap != null;
            hyperKycConfig = flHyperKyc.getHyperKycConfigFromMap(hyperKycConfigMap);
            Intent contractIntent = contract.createIntent(activity, hyperKycConfig);
            activity.startActivityForResult(contractIntent, FLHyperKycConstants.REQUEST_CODE_HYPERKYC_LAUNCH);
        } else {
            result.notImplemented();
        }
    };

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        this.context = flutterPluginBinding.getApplicationContext();
        hyperKycChannel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), FLHyperKycConstants.HYPERKYC_CHANNEL_NAME);
        hyperKycChannel.setMethodCallHandler(hyperKycMethodCallHandler);
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        hyperKycChannel.setMethodCallHandler(null);
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        this.activity = binding.getActivity();
        binding.addActivityResultListener(this);
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
        // Use this function only when necessary
    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
        // Use this function only when necessary
    }

    @Override
    public void onDetachedFromActivity() {
        // Use this function only when necessary
    }

    @Override
    public boolean onActivityResult(int requestCode, int resultCode, Intent data) {
        switch (requestCode) {
            case FLHyperKycConstants.REQUEST_CODE_HYPERKYC_LAUNCH:
                handleHyperKycResultData(resultCode, data);
                break;
            default:
                return false;
        }
        return true;
    }

    private void handleHyperKycResultData(int resultCode, Intent data) {
        if (data != null) {
            HyperKycResult hyperKycResult = contract.parseResult(resultCode, data);
            Map<String, Object> resultMap = processHyperKycResult(hyperKycResult);
            pendingResult.success(resultMap);
        }
    }

    private Map<String, Object> processHyperKycResult(HyperKycResult hyperKycResult) {
        Map<String, Object> resultMap = new HashMap<>();
        Map<String, Object> kycData = new HashMap<>();

        HyperKycData hyperKycData = hyperKycResult.getData();
        if (hyperKycData != null) {

            //  Add CountryResult to kycData
            HyperKycData.CountryResult countryResult = hyperKycData.countryResult();
            if (countryResult != null) {
                Map<String, Object> countryResultMap = new HashMap<>();
                countryResultMap.put(FLHyperKycConstants.MODULE_ID, FLHyperKycHelper.getStringFromProperty(countryResult.tag()));
                countryResultMap.put(FLHyperKycConstants.COUNTRY_ID, FLHyperKycHelper.getStringFromProperty(countryResult.countryId()));
                countryResultMap.put(FLHyperKycConstants.COUNTRY_NAME, FLHyperKycHelper.getStringFromProperty(countryResult.countryName()));
                countryResultMap.put(FLHyperKycConstants.REGION, FLHyperKycHelper.getStringFromProperty(countryResult.region()));
                countryResultMap.put(FLHyperKycConstants.BASE_URL, FLHyperKycHelper.getStringFromProperty(countryResult.baseUrl()));
//                countryResultMap.put(FLHyperKycConstants.VARIABLES, FLHyperKycHelper.getVariablesString(countryResult.variables()));

                kycData.put(FLHyperKycConstants.COUNTRY_RESULT, countryResultMap);
            } else {
                kycData.put(FLHyperKycConstants.COUNTRY_RESULT, new HashMap<>());
            }

            //  Add DocResultList to kycData
            List<Map<String, Object>> docResultMapList = new ArrayList<>();
            List<HyperKycData.DocResult> docResultList = hyperKycData.docResultList();
            for (HyperKycData.DocResult docResult : docResultList) {
                Map<String, Object> docResultMap = new HashMap<>();
                docResultMap.put(FLHyperKycConstants.MODULE_ID, FLHyperKycHelper.getStringFromProperty(docResult.tag()));
                docResultMap.put(FLHyperKycConstants.DOCUMENT_ID, FLHyperKycHelper.getStringFromProperty(docResult.documentId()));

                List<Map<String, Object>> docDataMapList = new ArrayList<>();
                List<HyperKycData.DocData> docDataList = docResult.docDataList();
                for (HyperKycData.DocData docData : docDataList) {
                    Map<String, Object> docDataMap = new HashMap<>();
                    docDataMap.put(FLHyperKycConstants.SIDE, FLHyperKycHelper.getStringFromProperty(docData.side()));
                    docDataMap.put(FLHyperKycConstants.DOC_IMAGE_PATH, FLHyperKycHelper.getStringFromProperty(docData.docImagePath()));
                    docDataMap.put(FLHyperKycConstants.ACTION, FLHyperKycHelper.getStringFromProperty(docData.action()));

                    Map<String, String> responseHeadersMap = docData.responseHeaders();
                    if (responseHeadersMap != null) {
                        docDataMap.put(FLHyperKycConstants.RESPONSE_HEADERS, FLHyperKycHelper.getResponseHeadersJsonString(new HashMap<>(responseHeadersMap)));
                    } else {
                        docDataMap.put(FLHyperKycConstants.RESPONSE_HEADERS, FLHyperKycConstants.EMPTY_JSON_STRING);
                    }

                    BaseResponse<DocCaptureApiDetail> responseResult = docData.responseBody();
                    docDataMap.put(FLHyperKycConstants.RESPONSE_RESULT, FLHyperKycHelper.getResponseResultJsonString(responseResult, null, null));

                    docDataMapList.add(docDataMap);
                }
                docResultMap.put(FLHyperKycConstants.DOC_DATA_LIST, docDataMapList);
//                docResultMap.put(FLHyperKycConstants.VARIABLES, FLHyperKycHelper.getVariablesString(docResult.variables()));
                docResultMapList.add(docResultMap);
            }
            kycData.put(FLHyperKycConstants.DOC_RESULT_LIST, docResultMapList);

            //  Add FaceResult to kycData
            HyperKycData.FaceResult faceResult = hyperKycData.faceResult();
            if (faceResult != null) {
                Map<String, Object> faceResultMap = new HashMap<>();
                faceResultMap.put(FLHyperKycConstants.MODULE_ID, FLHyperKycHelper.getStringFromProperty(faceResult.tag()));

                HyperKycData.FaceData faceData = faceResult.faceData();
                Map<String, Object> faceDataMap = new HashMap<>();
                faceDataMap.put(FLHyperKycConstants.CROPPED_FACE_IMAGE_PATH, FLHyperKycHelper.getStringFromProperty(faceData.croppedFaceImagePath()));
                faceDataMap.put(FLHyperKycConstants.FULL_FACE_IMAGE_PATH, FLHyperKycHelper.getStringFromProperty(faceData.fullFaceImagePath()));
                faceDataMap.put(FLHyperKycConstants.VIDEO_PATH, FLHyperKycHelper.getStringFromProperty(faceData.videoPath()));
                faceDataMap.put(FLHyperKycConstants.ACTION, FLHyperKycHelper.getStringFromProperty(faceData.action()));

                Map<String, String> responseHeadersMap = faceData.responseHeaders();
                if (responseHeadersMap != null) {
                    faceDataMap.put(FLHyperKycConstants.RESPONSE_HEADERS, FLHyperKycHelper.getResponseHeadersJsonString(new HashMap<>(responseHeadersMap)));
                } else {
                    faceDataMap.put(FLHyperKycConstants.RESPONSE_HEADERS, FLHyperKycConstants.EMPTY_JSON_STRING);
                }

                BaseResponse<FaceCaptureApiDetail> responseResult = faceData.responseBody();
                faceDataMap.put(FLHyperKycConstants.RESPONSE_RESULT, FLHyperKycHelper.getResponseResultJsonString(null, responseResult, null));

                faceResultMap.put(FLHyperKycConstants.FACE_DATA, faceDataMap);
//                faceResultMap.put(FLHyperKycConstants.VARIABLES, FLHyperKycHelper.getVariablesString(faceResult.variables()));

                kycData.put(FLHyperKycConstants.FACE_RESULT, faceResultMap);
            } else {
                kycData.put(FLHyperKycConstants.FACE_RESULT, new HashMap<>());
            }


            //  Add APIResultList to kycData
            List<Map<String, Object>> apiResultMapList = new ArrayList<>();
            List<HyperKycData.APIResult> apiResultList = hyperKycData.apiResultList();
            for (HyperKycData.APIResult apiResult : apiResultList) {
                Map<String, Object> apiResultMap = new HashMap<>();
                apiResultMap.put(FLHyperKycConstants.MODULE_ID, FLHyperKycHelper.getStringFromProperty(apiResult.tag()));

                HyperKycData.APIData apiData = apiResult.apiData();
                Map<String, Object> apiDataMap = new HashMap<>();
                apiDataMap.put(FLHyperKycConstants.RESPONSE_CODE, FLHyperKycHelper.getStringFromProperty(apiData.statusCode() != null ? String.valueOf(apiData.statusCode()) : FLHyperKycConstants.EMPTY_STRING));
                apiDataMap.put(FLHyperKycConstants.RESPONSE_BODY_RAW, FLHyperKycHelper.getStringFromProperty(apiData.responseBodyRaw()));

                Map<String, List<String>> responseHeaders = apiData.responseHeaders();
                if (responseHeaders != null) {
                    apiDataMap.put(FLHyperKycConstants.RESPONSE_HEADERS, FLHyperKycHelper.getResponseHeadersJsonString(new HashMap<>(responseHeaders)));
                } else {
                    apiDataMap.put(FLHyperKycConstants.RESPONSE_HEADERS, FLHyperKycConstants.EMPTY_JSON_STRING);
                }
                apiResultMap.put(FLHyperKycConstants.API_DATA, apiDataMap);
//                apiResultMap.put(FLHyperKycConstants.VARIABLES, FLHyperKycHelper.getVariablesString(apiResult.variables()));

                apiResultMapList.add(apiResultMap);
            }
            kycData.put(FLHyperKycConstants.API_RESULT_LIST, apiResultMapList);

            //  Add FormResultList to kycData
            List<Map<String, Object>> formResultMapList = new ArrayList<>();
            List<HyperKycData.FormResult> formResultList = hyperKycData.formResultList();
            for (HyperKycData.FormResult formResult : formResultList) {
                Map<String, Object> formResultMap = new HashMap<>();
                formResultMap.put(FLHyperKycConstants.MODULE_ID, FLHyperKycHelper.getStringFromProperty(formResult.tag()));

                formResultMapList.add(formResultMap);
            }
            kycData.put(FLHyperKycConstants.FORM_RESULT_LIST, formResultMapList);

            //  Add WebviewResultList to kycData
            List<Map<String, Object>> webviewResultMapList = new ArrayList<>();
            List<HyperKycData.WebviewResult> webviewResultList = hyperKycData.webviewResultList();
            for (HyperKycData.WebviewResult webviewResult : webviewResultList) {
                Map<String, Object> webviewResultMap = new HashMap<>();
                webviewResultMap.put(FLHyperKycConstants.MODULE_ID, FLHyperKycHelper.getStringFromProperty(webviewResult.tag()));

                HyperKycData.WebviewData webviewData = webviewResult.webviewData();
                Map<String, Object> webviewDataMap = new HashMap<>();
                webviewDataMap.put(FLHyperKycConstants.RAW_DATA, FLHyperKycHelper.getStringFromProperty(webviewData.rawData()));
                webviewResultMap.put(FLHyperKycConstants.WEBVIEW_DATA, webviewDataMap);

                webviewResultMapList.add(webviewResultMap);
            }
            kycData.put(FLHyperKycConstants.WEBVIEW_RESULT_LIST, webviewResultMapList);

        } else {
            kycData.put(FLHyperKycConstants.COUNTRY_RESULT, new HashMap<>());
            kycData.put(FLHyperKycConstants.DOC_RESULT_LIST, new ArrayList<>());
            kycData.put(FLHyperKycConstants.FACE_RESULT, new HashMap<>());
            kycData.put(FLHyperKycConstants.API_RESULT_LIST, new HashMap<>());
            kycData.put(FLHyperKycConstants.FORM_RESULT_LIST, new HashMap<>());
            kycData.put(FLHyperKycConstants.WEBVIEW_RESULT_LIST, new HashMap<>());
        }
        resultMap.put(FLHyperKycConstants.HYPERKYC_DATA, kycData);
        resultMap.put(FLHyperKycConstants.REASON, "");
        resultMap.put(FLHyperKycConstants.APPLICATION_STATUS, "");
        if (hyperKycResult.getStatus() == HyperKycResult.Status.CANCELLED) {
            // user cancelled
            resultMap.put(FLHyperKycConstants.STATUS, "cancelled");
        } else if (hyperKycResult.getStatus() == HyperKycResult.Status.FAILURE) {
            // failure
            resultMap.put(FLHyperKycConstants.REASON, hyperKycResult.getReason());
            resultMap.put(FLHyperKycConstants.STATUS, "failure");
        } else if (hyperKycResult.getStatus() == HyperKycResult.Status.SUCCESS) {
            // success
            resultMap.put(FLHyperKycConstants.STATUS, "success");
            resultMap.put(FLHyperKycConstants.APPLICATION_STATUS, hyperKycResult.getApplicationStatus());
        }
        return resultMap;
    }
}
