import Flutter
import UIKit
import HyperKYC

public class SwiftHyperkycFlutterPlugin: NSObject, FlutterPlugin {
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        
        let hyperKycChannel = FlutterMethodChannel(name: FLHyperKycConstants.HYPERKYC_CHANNEL_NAME, binaryMessenger: registrar.messenger())
        
        hyperKycChannel.setMethodCallHandler({
            (call: FlutterMethodCall, result:@escaping FlutterResult) -> Void in
            if (call.method.caseInsensitiveCompare(FLHyperKycConstants.LAUNCH) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return
                }
                let hyperKycConfigArgument = args[FLHyperKycConstants.HYPERKYCCONFIG] as! [String : Any]
                let completionHandler: (_ hyperKycResult: HyperKycResult) -> Void = {hyperKycResult in
                    
                    //  MARK: Process HyperKycResult and obtain final resultMap
                    let resultMap = processHyperKycResult(hyperKycResult: hyperKycResult)
                    
                    //  MARK: Pass the final resultMap to Flutter plugin
                    result(resultMap)
                }
                
                //  MARK: Call launch() function
                if let topController = UIApplication.shared.keyWindow?.rootViewController{
                    FLHyperKyc.launch(context: topController, hyperKycConfigArgument: hyperKycConfigArgument, callback: completionHandler)
                }
            } else {
                result(FlutterMethodNotImplemented)
            }
        })
    }
}

func processHyperKycResult(hyperKycResult: HyperKycResult) -> [String: Any] {
    
    //  MARK: Add result, kycData, errorMessage
    let statusEnum = hyperKycResult.status! as Status
    var statusEnumString = ""
    var kycData = [String: Any]()
    var reason = ""
    var applicationStatus = ""
    
    //  MARK: Add resultEnum
    statusEnumString = String(describing: statusEnum)
    
    //  MARK: Add kycData
    if let hyperKycData = hyperKycResult.hyperKYCData {
        
        //  MARK: Add countryResult to kycData
        if let countryResult = hyperKycData.countryResult {
            let countryResultDictionary = [
                FLHyperKycConstants.MODULE_ID : FLHyperKycHelper.getStringFromProperty(property: countryResult.moduleId),
                FLHyperKycConstants.COUNTRY_ID : FLHyperKycHelper.getStringFromProperty(property: countryResult.id),
                FLHyperKycConstants.COUNTRY_NAME : FLHyperKycHelper.getStringFromProperty(property: countryResult.name),
                FLHyperKycConstants.REGION : FLHyperKycHelper.getStringFromProperty(property: countryResult.region),
                FLHyperKycConstants.BASE_URL : FLHyperKycHelper.getStringFromProperty(property: countryResult.baseUrl),
                //                FLHyperKycConstants.VARIABLES : FLHyperKycHelper.getVariablesString(variablesDictionary: countryResult.variables),
            ]
            kycData[FLHyperKycConstants.COUNTRY_RESULT] = countryResultDictionary
        } else {
            kycData[FLHyperKycConstants.COUNTRY_RESULT] = [String: Any]()
        }
        
        //  MARK: Add docResult to kycData
        var docResultList : [[String: Any]] = []
        for docResult in hyperKycData.docResultList {
            var docResultDictionary : [String: Any] = [
                FLHyperKycConstants.MODULE_ID : FLHyperKycHelper.getStringFromProperty(property: docResult.moduleId),
                FLHyperKycConstants.DOCUMENT_ID : FLHyperKycHelper.getStringFromProperty(property: docResult.documentId)
            ]
            
            var docDataList: [[String: Any]] = []
            for docData in docResult.docDataList {
                var docDataDictionary = [
                    FLHyperKycConstants.SIDE : FLHyperKycHelper.getStringFromProperty(property: docData.side),
                    FLHyperKycConstants.DOC_IMAGE_PATH : FLHyperKycHelper.getStringFromProperty(property: docData.docImagePath),
                    FLHyperKycConstants.ACTION : FLHyperKycHelper.getStringFromProperty(property: docData.action),
                ]
                if let responseHeaders = docData.responseHeaders  {
                    docDataDictionary[FLHyperKycConstants.RESPONSE_HEADERS] = FLHyperKycHelper.getResponseHeadersJsonString(responseHeaders: responseHeaders)
                } else {
                    docDataDictionary[FLHyperKycConstants.RESPONSE_HEADERS] = FLHyperKycConstants.EMPTY_JSON_STRING
                }
                if let responseResult = docData.responseResult {
                    docDataDictionary[FLHyperKycConstants.RESPONSE_RESULT] = FLHyperKycHelper.getResponseResultJsonString(docDataResponseResult: responseResult, faceDataResponseResult: nil, faceMatchDataResponseResult: nil)
                } else {
                    docDataDictionary[FLHyperKycConstants.RESPONSE_RESULT] = FLHyperKycConstants.EMPTY_JSON_STRING
                }
                docDataList.append(docDataDictionary as [String : Any])
            }
            docResultDictionary[FLHyperKycConstants.DOC_DATA_LIST] = docDataList
            //            docResultDictionary[FLHyperKycConstants.VARIABLES] = FLHyperKycHelper.getVariablesString(variablesDictionary: docResult.variables)
            
            docResultList.append(docResultDictionary)
        }
        kycData[FLHyperKycConstants.DOC_RESULT_LIST] = docResultList
        
        //  MARK: Add faceResult to kycData
        if let faceResult = hyperKycData.faceResult {
            
            var faceResultDictionary : [String: Any] = [
                FLHyperKycConstants.MODULE_ID : FLHyperKycHelper.getStringFromProperty(property: faceResult.moduleId)
            ]
            
            let faceData = faceResult.faceData
            var faceDataDictionary : [String: Any] = [
                FLHyperKycConstants.CROPPED_FACE_IMAGE_PATH : FLHyperKycHelper.getStringFromProperty(property: faceData.croppedFaceImagePath),
                FLHyperKycConstants.FULL_FACE_IMAGE_PATH : FLHyperKycHelper.getStringFromProperty(property: faceData.fullFaceImagePath),
                FLHyperKycConstants.VIDEO_PATH : FLHyperKycHelper.getStringFromProperty(property: faceData.videoPath),
                FLHyperKycConstants.ACTION : FLHyperKycHelper.getStringFromProperty(property: faceData.action),
            ]
            if let responseHeaders = faceData.responseHeaders  {
                faceDataDictionary[FLHyperKycConstants.RESPONSE_HEADERS] = FLHyperKycHelper.getResponseHeadersJsonString(responseHeaders: responseHeaders)
            } else {
                faceDataDictionary[FLHyperKycConstants.RESPONSE_HEADERS] = FLHyperKycConstants.EMPTY_JSON_STRING
            }
            if let responseResult = faceData.responseResult {
                faceDataDictionary[FLHyperKycConstants.RESPONSE_RESULT] = FLHyperKycHelper.getResponseResultJsonString(docDataResponseResult: nil, faceDataResponseResult: responseResult, faceMatchDataResponseResult: nil)
            } else {
                faceDataDictionary[FLHyperKycConstants.RESPONSE_RESULT] = FLHyperKycConstants.EMPTY_JSON_STRING
            }
            faceResultDictionary[FLHyperKycConstants.FACE_DATA] = faceDataDictionary
            
            
            //            faceResultDictionary[FLHyperKycConstants.VARIABLES] = FLHyperKycHelper.getVariablesString(variablesDictionary: faceResult.variables)
            
            kycData[FLHyperKycConstants.FACE_RESULT] = faceResultDictionary
        } else {
            kycData[FLHyperKycConstants.FACE_RESULT] = [String: Any]()
        }
        
        //  MARK: Add apiResult to kycData
        var apiResultList: [[String : Any]] = []
        for apiResult in hyperKycData.apiResultList {
            var apiResultDictionary : [String : Any] = [
                FLHyperKycConstants.MODULE_ID : FLHyperKycHelper.getStringFromProperty(property: apiResult.moduleId),
            ]
            
            let apiData = apiResult.apiData
            var apiDataDictionary = [
                FLHyperKycConstants.RESPONSE_CODE : FLHyperKycHelper.getStringFromProperty(property: String(apiData.responseCode ?? -100)),
            ]
            if let responseHeaders = apiData.responseHeaders  {
                apiDataDictionary[FLHyperKycConstants.RESPONSE_HEADERS] = FLHyperKycHelper.getResponseHeadersJsonString(responseHeaders: responseHeaders)
            } else {
                apiDataDictionary[FLHyperKycConstants.RESPONSE_HEADERS] = FLHyperKycConstants.EMPTY_JSON_STRING
            }
            if let responseResult = apiData.responseBodyRaw {
                apiDataDictionary[FLHyperKycConstants.RESPONSE_BODY_RAW] = FLHyperKycHelper.getResponseHeadersJsonString(responseHeaders: responseResult)
            } else {
                apiDataDictionary[FLHyperKycConstants.RESPONSE_BODY_RAW] = FLHyperKycConstants.EMPTY_JSON_STRING
            }
            apiResultDictionary[FLHyperKycConstants.API_DATA] = apiDataDictionary
            //            apiResultDictionary[FLHyperKycConstants.VARIABLES] = FLHyperKycHelper.getVariablesString(variablesDictionary: apiResult.variables)
            
            apiResultList.append(apiResultDictionary)
        }
        kycData[FLHyperKycConstants.API_RESULT_LIST] = apiResultList
        
        //  MARK: Add formResult to kycData
        var formResultList: [[String : Any]] = []
        for formResult in hyperKycData.formResultList {
            let formResultDictionary : [String : Any] = [
                FLHyperKycConstants.MODULE_ID : FLHyperKycHelper.getStringFromProperty(property: formResult.moduleId),
            ]
            formResultList.append(formResultDictionary)
        }
        kycData[FLHyperKycConstants.FORM_RESULT_LIST] = formResultList
        
        //  MARK: Add webviewResult to kycData
        var webviewResultList: [[String : Any]] = []
        for webviewResult in hyperKycData.webviewResultList {
            var webviewResultDictionary : [String : Any] = [
                FLHyperKycConstants.MODULE_ID : FLHyperKycHelper.getStringFromProperty(property: webviewResult.moduleId),
            ]
            
            let webviewData = webviewResult.webviewData
            let webviewDataDictionary = [
                FLHyperKycConstants.RAW_DATA : FLHyperKycHelper.getStringFromProperty(property: webviewData.rawData),
            ]
            
            webviewResultDictionary[FLHyperKycConstants.WEBVIEW_DATA] = webviewDataDictionary
            
            webviewResultList.append(webviewResultDictionary)
        }
        kycData[FLHyperKycConstants.WEBVIEW_RESULT_LIST] = webviewResultList
        
    } else {
        kycData[FLHyperKycConstants.COUNTRY_RESULT] = [String: Any]()
        kycData[FLHyperKycConstants.DOC_RESULT_LIST] = []
        kycData[FLHyperKycConstants.FACE_RESULT] = [String : Any]()
        kycData[FLHyperKycConstants.API_RESULT_LIST] = []
        kycData[FLHyperKycConstants.FORM_RESULT_LIST] = []
        kycData[FLHyperKycConstants.WEBVIEW_RESULT_LIST] = []
    }
    
    //  MARK: Add errorMessage
    if let errorMsg = hyperKycResult.reason {
        reason = errorMsg
    }
    
    if let applStatus = hyperKycResult.applicationStatus {
        applicationStatus = applStatus
    }
    
    //  MARK: Create the final result map which will include statusEnum, applicationStatus, kycData and errorMessage
    let resultMap = [
        FLHyperKycConstants.STATUS : statusEnumString,
        FLHyperKycConstants.HYPERKYC_DATA : kycData,
        FLHyperKycConstants.REASON : reason,
        FLHyperKycConstants.APPLICATION_STATUS : applicationStatus,
    ] as [String : Any]
    
    return resultMap
}
