import 'package:hyperkyc_flutter/hyperkyc_flutter_constants.dart';

/// ApiResult contains ApiData
class ApiResult {
  /// moduleId for which this apiResult belongs to
  final String? moduleId;

  /// ApiData
  final ApiData? apiData;

  /// variables related to ApiResult
  // final String? variables;

  /// Constructor
  ApiResult({
    this.moduleId,
    this.apiData,
    // this.variables,
  });

  /// Convert ApiResult object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.moduleId: this.moduleId,
      HyperKycConstants.apiData: this.apiData,
      // HyperKycConstants.variables: this.variables,
    };
  }

  /// Convert Map object to ApiResult object
  factory ApiResult.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return ApiResult();
    }
    return ApiResult(
      moduleId: map[HyperKycConstants.moduleId] as String?,
      apiData: ApiData.fromMap(map[HyperKycConstants.apiData]),
      // variables: map[HyperKycConstants.variables] as String?,
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'ApiResult{moduleId: $moduleId, apiData: $apiData}';
  }
}

/// ApiData holds the responses of the performed api call
class ApiData {
  /// Response code of the porformed api call
  final int? responseCode;

  /// Response body (raw) of the performed api call
  final String? responseBodyRaw;

  /// Response headers of the performed api call
  final String? responseHeaders;

  /// Constructor
  ApiData({
    this.responseCode,
    this.responseBodyRaw,
    this.responseHeaders,
  });

  /// Convert ApiData object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.responseCode: this.responseCode,
      HyperKycConstants.responseBodyRaw: this.responseBodyRaw,
      HyperKycConstants.responseHeaders: this.responseHeaders,
    };
  }

  /// Convert Map to ApiData
  factory ApiData.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return ApiData();
    }
    return ApiData(
      responseCode: int.parse(map[HyperKycConstants.responseCode] ?? -100),
      responseBodyRaw: map[HyperKycConstants.responseBodyRaw] as String?,
      responseHeaders: map[HyperKycConstants.responseHeaders] as String?,
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'ApiData{responseCode: $responseCode, responseBodyRaw: $responseBodyRaw, responseHeaders: $responseHeaders}';
  }
}
