import 'hyperkyc_flutter_constants.dart';

/// [HyperKycConfig] is used to launch HyperKyc
/// Create an instance of HyperKycConfig and call launch with it to start the workflow
class HyperKycConfig {
  /// [appId] to launch HyperKYC : Get this from Hyperverge team
  String? appId;

  /// [appKey] to launch HyperKYC : Get this from Hyperverge team
  String? appKey;

  /// [accessToken] to launch HyperKYC : Please contact Hyperverge team to understand how to generate HV accessToken
  String? accessToken;

  /// [workflowId] to launch HyperKYC : Each workflow that you have created in Hyperverge portal has a workflowId
  String? workflowId;

  /// [transactionId] to launch HyperKYC : Please contact Hyperverge team to understand more on sessions and transactionId
  String? transactionId;

  /// CustomInputs [inputs] to launch HyperKYC : If workflow requires any custom inputs, please pass those custom inputs here
  Map<String, String>? inputs;

  /// Create HyperKycConfig instance from accessToken
  HyperKycConfig.fromAccessToken({
    required this.accessToken,
    required this.workflowId,
    required this.transactionId,
  });

  /// Create HyperKycConfig instance from appId, appKey
  HyperKycConfig.fromAppIdAppKey({
    required this.appId,
    required this.appKey,
    required this.workflowId,
    required this.transactionId,
  });

  /// Use [setInputs] to pass customInputs which are required by the workflow
  void setInputs({required Map<String, String> inputs}) {
    this.inputs = inputs;
  }

  /// Overriden [toString] method
  @override
  String toString() {
    return 'HyperKycConfig{appId: $appId, appKey: $appKey, accessToken: $accessToken, workflowId: $workflowId, transactionId: $transactionId, inputs: $inputs}';
  }

  /// Convert HyperKycConfig object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.appId: this.appId,
      HyperKycConstants.appKey: this.appKey,
      HyperKycConstants.accessToken: this.accessToken,
      HyperKycConstants.workflowId: this.workflowId,
      HyperKycConstants.transactionId: this.transactionId,
      HyperKycConstants.inputs: this.inputs,
    };
  }
}
