import 'package:hyperkyc_flutter/hyperkyc_api_result.dart';
import 'package:hyperkyc_flutter/hyperkyc_doc_result.dart';
import 'package:hyperkyc_flutter/hyperkyc_face_result.dart';

import 'hyperkyc_country_result.dart';
import 'hyperkyc_flutter_constants.dart';
import 'hyperkyc_form_result.dart';
import 'hyperkyc_webview_result.dart';

/// HyperKycData will contain partial or the final result data based on the result state.
class HyperKycData {
  /// Country result - country name, id, region, baseUrl, etc
  final CountryResult? countryResult;

  /// List of document capture and OCR results
  final List<DocResult?>? docResultList;

  /// Face capture and liveness result
  final FaceResult? faceResult;

  /// List of API call results
  final List<ApiResult?>? apiResultList;

  /// List of Form results
  final List<FormResult?>? formResultList;

  /// List of Webview results
  final List<WebviewResult?>? webviewResultList;

  /// Constructor
  HyperKycData({
    this.countryResult,
    this.docResultList,
    this.faceResult,
    this.apiResultList,
    this.formResultList,
    this.webviewResultList,
  });

  /// Convert HyperKycData object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.countryResult: this.countryResult,
      HyperKycConstants.docResultList: this.docResultList,
      HyperKycConstants.faceResult: this.faceResult,
      HyperKycConstants.apiResultList: this.apiResultList,
      HyperKycConstants.formResultList: this.formResultList,
      HyperKycConstants.webviewResultList: this.webviewResultList,
    };
  }

  /// Convert Map to HyperKycData object
  factory HyperKycData.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return HyperKycData();
    }

    List<DocResult> docResultList = List.empty(growable: true);
    List docResultListFromMap = map[HyperKycConstants.docResultList] ?? [];
    for (var docResultMap in docResultListFromMap) {
      DocResult docResult = DocResult.fromMap(docResultMap ?? {});
      docResultList.add(docResult);
    }

    List<ApiResult> apiDataResultList = List.empty(growable: true);
    List apiResultListFromMap = map[HyperKycConstants.apiResultList] ?? [];
    for (var apiResultMap in apiResultListFromMap) {
      ApiResult apiResult = ApiResult.fromMap(apiResultMap ?? {});
      apiDataResultList.add(apiResult);
    }

    List<FormResult> formResultList = List.empty(growable: true);
    List formResultListFromMap = map[HyperKycConstants.formResultList] ?? [];
    for (var formResultMap in formResultListFromMap) {
      FormResult formResult = FormResult.fromMap(formResultMap ?? {});
      formResultList.add(formResult);
    }

    List<WebviewResult> webviewResultList = List.empty(growable: true);
    List webviewResultListFromMap =
        map[HyperKycConstants.webviewResultList] ?? [];
    for (var webviewResultMap in webviewResultListFromMap) {
      WebviewResult webviewResult =
          WebviewResult.fromMap(webviewResultMap ?? {});
      webviewResultList.add(webviewResult);
    }

    return HyperKycData(
      countryResult:
          CountryResult.fromMap(map[HyperKycConstants.countryResult] ?? {}),
      docResultList: docResultList,
      faceResult: FaceResult.fromMap(map[HyperKycConstants.faceResult] ?? {}),
      apiResultList: apiDataResultList,
      formResultList: formResultList,
      webviewResultList: webviewResultList,
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'HyperKycData{countryResult: $countryResult, docResultList: $docResultList, faceResult: $faceResult, apiResultList: $apiResultList, formResultList: $formResultList, webviewResultList: $webviewResultList}';
  }
}
