import 'package:hyperkyc_flutter/hyperkyc_flutter_constants.dart';

/// WebviewResult
class WebviewResult {
  /// moduleId for which this webviewResult belongs to
  final String? moduleId;

  /// WebviewData
  final WebviewData? webviewData;

  /// Constructor
  WebviewResult({
    this.moduleId,
    this.webviewData,
  });

  /// Convert WebviewResult object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.moduleId: this.moduleId,
      HyperKycConstants.webviewData: this.webviewData,
    };
  }

  /// Convert Map to WebviewResult object
  factory WebviewResult.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return WebviewResult();
    }
    return WebviewResult(
      moduleId: map[HyperKycConstants.moduleId] as String?,
      webviewData:
          WebviewData.fromMap(map[HyperKycConstants.webviewData] ?? {}),
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'WebviewResult{moduleId: $moduleId, webviewData: $webviewData}';
  }
}

/// WebviewData
class WebviewData {
  /// rawData received from webview
  final String? rawData;

  /// Constructor
  WebviewData({
    this.rawData,
  });

  /// Convert WebviewData object to Map
  Map<String, dynamic> toMap() {
    return {
      'rawData': this.rawData,
    };
  }

  /// Convert Map to WebviewData object
  factory WebviewData.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return WebviewData();
    }
    return WebviewData(
      rawData: map['rawData'] as String?,
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'WebviewData{rawData: $rawData}';
  }
}
